/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_TRPCLATSERVICE_TRPCLATSERVICE_SKELETON_H
#define JAC_AP_TRPCLATSERVICE_TRPCLATSERVICE_SKELETON_H

#include "ara/com/internal/skeleton/skeleton_adapter.h"
#include "ara/com/internal/skeleton/event_adapter.h"
#include "ara/com/internal/skeleton/field_adapter.h"
#include "ara/com/internal/skeleton/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/trpclatservice/trpclatservice_common.h"
#include <cstdint>

namespace jac {
namespace ap {
namespace trpclatservice {
namespace skeleton {
namespace events
{
    using TRPCLat2CANEvent = ara::com::internal::skeleton::event::EventAdapter<::SWC2CAN>;
    static constexpr ara::com::internal::EntityId TRPCLatServiceTRPCLat2CANEventId = 30378U; //TRPCLat2CANEvent_event_hash
}

namespace methods
{
}

namespace fields
{
}

class TRPCLatServiceSkeleton {
private:
    std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> skeletonAdapter;
    void ConstructSkeleton(const ara::com::MethodCallProcessingMode mode)
    {
        if (mode == ara::com::MethodCallProcessingMode::kEvent) {
            if (!(skeletonAdapter->SetMethodThreadNumber(skeletonAdapter->GetMethodThreadNumber(0U), 1024U))) {
#ifndef NOT_SUPPORT_EXCEPTIONS
                ara::core::ErrorCode errorcode(ara::com::ComErrc::kNetworkBindingFailure);
                throw ara::com::ComException(std::move(errorcode));
#else
                std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
            }
        }
        const ara::core::Result<void> resultTRPCLat2CANEvent = TRPCLat2CANEvent.Initialize();
        ThrowError(resultTRPCLat2CANEvent);
    }

    TRPCLatServiceSkeleton& operator=(const TRPCLatServiceSkeleton&) = delete;

    static void ThrowError(const ara::core::Result<void>& result)
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
        }
    }
public:
    class ConstructionToken {
    public:
        explicit ConstructionToken(const ara::com::InstanceIdentifier& instanceId,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, instanceId, mode)),
              TRPCLat2CANEvent(ptr->GetSkeleton(), events::TRPCLatServiceTRPCLat2CANEventId, ::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, instanceId){
        }

        explicit ConstructionToken(const ara::core::InstanceSpecifier& instanceSpec,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, instanceSpec, mode)),
              TRPCLat2CANEvent(ptr->GetSkeleton(), events::TRPCLatServiceTRPCLat2CANEventId, ::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, instanceSpec){
        }

        explicit ConstructionToken(const ara::com::InstanceIdentifierContainer instanceContainer,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, instanceContainer, mode)),
              TRPCLat2CANEvent(ptr->GetSkeleton(), events::TRPCLatServiceTRPCLat2CANEventId, ::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, instanceContainer){
        }

        ConstructionToken(ConstructionToken&& other)
            : processMode(std::move(other.processMode)),
              ptr(std::move(other.ptr)),
              TRPCLat2CANEvent(std::move(other.TRPCLat2CANEvent)){
        }
        ConstructionToken& operator=(ConstructionToken && other)
        {
            if (&other != this) {
                processMode = std::move(other.processMode);
                ptr = std::move(other.ptr);
                TRPCLat2CANEvent = std::move(other.TRPCLat2CANEvent);
            }
            return *this;
        }
        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> GetSkeletonAdapter()
        {
            return std::move(ptr);
        }
        events::TRPCLat2CANEvent GetTRPCLat2CANEvent()
        {
            return std::move(TRPCLat2CANEvent);
        }
        ara::core::Result<void> Initialize()
        {
            ara::core::Result<void> initResult;
            do {
                if (processMode == ara::com::MethodCallProcessingMode::kEvent) {
                    if(!ptr->SetMethodThreadNumber(ptr->GetMethodThreadNumber(0U), 1024U)) {
                        ara::core::ErrorCode errorCode(ara::com::ComErrc::kNetworkBindingFailure);
                        initResult.EmplaceError(errorCode);
                        break;
                    }
                }
                initResult = TRPCLat2CANEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
            } while(false);

            return initResult;
        }
    private:
        ara::com::MethodCallProcessingMode processMode;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> ptr;
        events::TRPCLat2CANEvent TRPCLat2CANEvent;
    };
    explicit TRPCLatServiceSkeleton(const ara::com::InstanceIdentifier& instanceId,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        : skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, instanceId, mode)),
          TRPCLat2CANEvent(skeletonAdapter->GetSkeleton(), events::TRPCLatServiceTRPCLat2CANEventId, ::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, instanceId){
        ConstructSkeleton(mode);
    }

    explicit TRPCLatServiceSkeleton(const ara::core::InstanceSpecifier& instanceSpec,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, instanceSpec, mode)),
          TRPCLat2CANEvent(skeletonAdapter->GetSkeleton(), events::TRPCLatServiceTRPCLat2CANEventId, ::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, instanceSpec){
        ConstructSkeleton(mode);
    }

    explicit TRPCLatServiceSkeleton(const ara::com::InstanceIdentifierContainer instanceContainer,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, instanceContainer, mode)),
          TRPCLat2CANEvent(skeletonAdapter->GetSkeleton(), events::TRPCLatServiceTRPCLat2CANEventId, ::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, instanceContainer){
        ConstructSkeleton(mode);
    }

    TRPCLatServiceSkeleton(const TRPCLatServiceSkeleton&) = delete;

    TRPCLatServiceSkeleton(TRPCLatServiceSkeleton&&) = default;
    TRPCLatServiceSkeleton& operator=(TRPCLatServiceSkeleton&&) = default;
    TRPCLatServiceSkeleton(ConstructionToken&& token) noexcept
        : skeletonAdapter(token.GetSkeletonAdapter()),
          TRPCLat2CANEvent(token.GetTRPCLat2CANEvent()){
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifier instanceId,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceId, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::core::InstanceSpecifier instanceSpec,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceSpec, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifierContainer instanceIdContainer,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceIdContainer, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    virtual ~TRPCLatServiceSkeleton()
    {
        StopOfferService();
    }

    void OfferService()
    {
        skeletonAdapter->RegisterE2EErrorHandler(&TRPCLatServiceSkeleton::E2EErrorHandler, *this);
        skeletonAdapter->OfferService();
    }
    void StopOfferService()
    {
        skeletonAdapter->StopOfferService();
    }
    ara::core::Future<bool> ProcessNextMethodCall()
    {
        return skeletonAdapter->ProcessNextMethodCall();
    }
    bool SetMethodThreadNumber(const std::uint16_t& number, const std::uint16_t& queueSize)
    {
        return skeletonAdapter->SetMethodThreadNumber(number, queueSize);
    }

    virtual void E2EErrorHandler(ara::com::e2e::E2EErrorCode, ara::com::e2e::DataID, ara::com::e2e::MessageCounter){}

    events::TRPCLat2CANEvent TRPCLat2CANEvent;
};
} // namespace skeleton
} // namespace trpclatservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_TRPCLATSERVICE_TRPCLATSERVICE_SKELETON_H
